@@ -724,9 +724,9 @@ rasqal_algebra_node_operator_as_counted_string(rasqal_algebra_node_operator op,
     op = RASQAL_ALGEBRA_OPERATOR_UNKNOWN;
 
   if(length_p)
-    *length_p = rasqal_algebra_node_operator_labels[(int)op].length;
+    *length_p = rasqal_algebra_node_operator_labels[RASQAL_GOOD_CAST(int, op)].length;
 
-  return rasqal_algebra_node_operator_labels[(int)op].label;
+  return rasqal_algebra_node_operator_labels[RASQAL_GOOD_CAST(int, op)].label;
 }
   
 
@@ -765,7 +765,7 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
   raptor_iostream_counted_string_write(op_string, op_length, iostr);
   raptor_iostream_counted_string_write("(\n", 2, iostr);
 
-  indent_delta = (unsigned int)op_length + 1;
+  indent_delta = RASQAL_GOOD_CAST(unsigned int, op_length) + 1;
   
   indent += indent_delta;
   rasqal_algebra_write_indent(iostr, indent);
@@ -1009,7 +1009,7 @@ rasqal_algebra_basic_graph_pattern_to_algebra(rasqal_query* query,
   if(fs)
     rasqal_free_expression(fs);
   
-  return node;
+  return NULL;
 }
 
 
@@ -1090,6 +1090,30 @@ rasqal_algebra_union_graph_pattern_to_algebra(rasqal_query* query,
 }
 
 
+/*
+ * rasqal_algebra_new_boolean_constant_expr:
+ * @query: query object
+ * @value: boolean constant
+ *
+ * INTERNAL - Create a new expression for a boolean constant (true/false)
+ *
+ * Return value: new expression or NULL on failure
+ */
+static rasqal_expression*
+rasqal_algebra_new_boolean_constant_expr(rasqal_query* query, int value)
+{
+  rasqal_literal *true_lit;
+  
+  true_lit = rasqal_new_boolean_literal(query->world, value);
+  if(!true_lit) {
+    RASQAL_DEBUG1("rasqal_new_boolean_literal() failed\n");
+    return NULL;
+  }
+  
+  return rasqal_new_literal_expression(query->world, true_lit);
+}
+
+
 static rasqal_algebra_node*
 rasqal_algebra_group_graph_pattern_to_algebra(rasqal_query* query,
                                               rasqal_graph_pattern* gp)
@@ -1167,24 +1191,14 @@ rasqal_algebra_group_graph_pattern_to_algebra(rasqal_query* query,
             goto fail;
           }
         } else  {
-          rasqal_literal *true_lit = NULL;
           rasqal_expression *true_expr = NULL;
 
-          true_lit = rasqal_new_boolean_literal(query->world, 1);
-          if(!true_lit) {
-            RASQAL_DEBUG1("rasqal_new_boolean_literal() failed\n");
+          true_expr = rasqal_algebra_new_boolean_constant_expr(query, 1);
+          if(!true_expr) {
             rasqal_free_algebra_node(anode);
-            goto fail;
+            return NULL;
           }
           
-          true_expr = rasqal_new_literal_expression(query->world, true_lit);
-          if(!true_expr) {
-            RASQAL_DEBUG1("rasqal_new_literal_expression() failed\n");
-            rasqal_free_algebra_node(anode);
-            goto fail;
-          }
-          true_lit = NULL; /* now owned by true_expr */
-
           /* G := LeftJoin(G, A, true) */
           gnode = rasqal_new_leftjoin_algebra_node(query, gnode, anode,
                                                    true_expr);
@@ -1410,6 +1424,12 @@ rasqal_algebra_graph_pattern_to_algebra(rasqal_query* query,
   }
 
 #if RASQAL_DEBUG > 1
+  if(gp) {
+    RASQAL_DEBUG1("Input gp:\n");
+    rasqal_graph_pattern_print(gp, stderr);
+    fputc('\n', stderr);
+  }
+
   if(node) {
     RASQAL_DEBUG1("Resulting node:\n");
     rasqal_algebra_node_print(node, stderr);
@@ -1655,7 +1675,7 @@ rasqal_algebra_extract_aggregate_expression_visit(void *user_data,
 
     v = rasqal_variables_table_add(ae->query->vars_table, 
                                    RASQAL_VARIABLE_TYPE_ANONYMOUS, 
-                                   (const unsigned char*)var_name, NULL);
+                                   RASQAL_GOOD_CAST(const unsigned char*, var_name), NULL);
     if(!v) {
       ae->error = 1;
       return 1;
