@@ -737,6 +737,8 @@ rasqal_expression_clear(rasqal_expression* e)
     case RASQAL_EXPR_SHA256:
     case RASQAL_EXPR_SHA384:
     case RASQAL_EXPR_SHA512:
+    case RASQAL_EXPR_UUID:
+    case RASQAL_EXPR_STRUUID:
       /* arg1 is optional for RASQAL_EXPR_BNODE */
       if(e->arg1)
         rasqal_free_expression(e->arg1);
@@ -957,6 +959,8 @@ rasqal_expression_visit(rasqal_expression* e,
     case RASQAL_EXPR_SHA256:
     case RASQAL_EXPR_SHA384:
     case RASQAL_EXPR_SHA512:
+    case RASQAL_EXPR_UUID:
+    case RASQAL_EXPR_STRUUID:
       /* arg1 is optional for RASQAL_EXPR_BNODE */
       return (e->arg1) ? rasqal_expression_visit(e->arg1, fn, user_data) : 0;
       break;
@@ -1105,7 +1109,9 @@ static const char* const rasqal_op_labels[RASQAL_EXPR_LAST+1]={
   "sha512",
   "strbefore",
   "strafter",
-  "replace"
+  "replace",
+  "uuid",
+  "struuid"
 };
 
 
@@ -1123,7 +1129,7 @@ rasqal_expression_op_label(rasqal_op op)
   if(op > RASQAL_EXPR_LAST)
     op = RASQAL_EXPR_UNKNOWN;
 
-  return rasqal_op_labels[(int)op];
+  return rasqal_op_labels[RASQAL_GOOD_CAST(int, op)];
 }
 
 
@@ -1300,6 +1306,8 @@ rasqal_expression_write(rasqal_expression* e, raptor_iostream* iostr)
     case RASQAL_EXPR_SHA256:
     case RASQAL_EXPR_SHA384:
     case RASQAL_EXPR_SHA512:
+    case RASQAL_EXPR_UUID:
+    case RASQAL_EXPR_STRUUID:
       raptor_iostream_counted_string_write("op ", 3, iostr);
       rasqal_expression_write_op(e, iostr);
       raptor_iostream_write_byte('(', iostr);
@@ -1528,6 +1536,8 @@ rasqal_expression_print(rasqal_expression* e, FILE* fh)
     case RASQAL_EXPR_SHA256:
     case RASQAL_EXPR_SHA384:
     case RASQAL_EXPR_SHA512:
+    case RASQAL_EXPR_UUID:
+    case RASQAL_EXPR_STRUUID:
       fputs("op ", fh);
       rasqal_expression_print_op(e, fh);
       fputc('(', fh);
@@ -1743,6 +1753,8 @@ rasqal_expression_is_constant(rasqal_expression* e)
     case RASQAL_EXPR_SHA256:
     case RASQAL_EXPR_SHA384:
     case RASQAL_EXPR_SHA512:
+    case RASQAL_EXPR_UUID:
+    case RASQAL_EXPR_STRUUID:
       /* arg1 is optional for RASQAL_EXPR_BNODE and result is always constant */
       result = (e->arg1) ? rasqal_expression_is_constant(e->arg1) : 1;
       break;
@@ -1832,7 +1844,7 @@ rasqal_expression_has_variable(void *user_data, rasqal_expression *e)
   if(!v)
     return 0;
   
-  if(!strcmp((const char*)v->name, (const char*)name))
+  if(!strcmp(RASQAL_GOOD_CAST(const char*, v->name), RASQAL_GOOD_CAST(const char*, name)))
     return 1;
 
   return 0;
@@ -2115,6 +2127,8 @@ rasqal_expression_compare(rasqal_expression* e1, rasqal_expression* e2,
     case RASQAL_EXPR_SHA256:
     case RASQAL_EXPR_SHA384:
     case RASQAL_EXPR_SHA512:
+    case RASQAL_EXPR_UUID:
+    case RASQAL_EXPR_STRUUID:
       /* arg1 is optional for RASQAL_EXPR_BNODE */
       rc = rasqal_expression_compare(e1->arg1, e2->arg1, flags, error_p);
       break;
