@@ -251,7 +251,7 @@ rasqal_builtin_agg_expression_execute_step(void* user_data,
   b = (rasqal_builtin_agg_expression_execute*)user_data;
 
   if(b->error)
-    return 1;
+    return b->error;
   
   if(b->expr->op == RASQAL_EXPR_COUNT) {
     /* COUNT(*) : counts every row (does not care about literals) */
@@ -283,7 +283,7 @@ rasqal_builtin_agg_expression_execute_step(void* user_data,
       const unsigned char* str;
       int error = 0;
       
-      str = (const unsigned char*)rasqal_literal_as_string_flags(l, 0, &error);
+      str = RASQAL_GOOD_CAST(const unsigned char*, rasqal_literal_as_string_flags(l, 0, &error));
 
       if(!error) {
         if(raptor_stringbuffer_length(b->sb))
@@ -327,12 +327,9 @@ rasqal_builtin_agg_expression_execute_step(void* user_data,
 
 #if RASQAL_DEBUG > 1
     RASQAL_DEBUG3("Aggregation step result %s (error=%d)\n", 
-                  (result ? (const char*)rasqal_literal_as_string(result) : "(NULL)"),
+                  (result ? RASQAL_GOOD_CAST(const char*, rasqal_literal_as_string(result)) : "(NULL)"),
                   b->error);
 #endif
-    
-    if(b->error)
-      break;
   }
   
   return b->error;
@@ -346,6 +343,9 @@ rasqal_builtin_agg_expression_execute_result(void* user_data)
 
   b = (rasqal_builtin_agg_expression_execute*)user_data;
 
+  if(b->error)
+    return NULL;
+
   if(b->expr->op == RASQAL_EXPR_COUNT) {
     rasqal_literal* result;
 
@@ -654,7 +654,7 @@ rasqal_aggregation_rowsource_read_row(rasqal_rowsource* rowsource,
         }
 
 #ifdef RASQAL_DEBUG
-        RASQAL_DEBUG1("Aggregation step over literals: ");
+        RASQAL_DEBUG2("Aggregation expr %d step over literals: ", i);
         raptor_sequence_print(seq, DEBUG_FH);
         fputc('\n', DEBUG_FH);
 #endif
@@ -667,8 +667,10 @@ rasqal_aggregation_rowsource_read_row(rasqal_rowsource* rowsource,
         if(!expr_data->map)
           raptor_free_sequence(seq);
 
-        if(error)
-          break;
+        if(error) {
+          RASQAL_DEBUG2("Aggregation expr %d returned error\n", i);
+          error = 0;
+        }
       }
     }
 
@@ -717,7 +719,7 @@ rasqal_aggregation_rowsource_read_row(rasqal_rowsource* rowsource,
       result = rasqal_builtin_agg_expression_execute_result(expr_data->agg_user_data);
   
 #ifdef RASQAL_DEBUG
-      RASQAL_DEBUG1("Aggregation ending group with result: ");
+      RASQAL_DEBUG2("Aggregation %d ending group with result: ", i);
       if(result)
         rasqal_literal_print(result, DEBUG_FH);
       else
@@ -795,7 +797,8 @@ static const rasqal_rowsource_handler rasqal_aggregation_rowsource_handler = {
  *
  * INTERNAL - Create a new rowsource for a aggregration
  *
- * The @rowsource becomes owned by the new rowsource.
+ * The @rowsource becomes owned by the new rowsource.  The @exprs_seq
+ * and @vars_seq are not. 
  *
  * For example with the SPARQL 1.1 example queries
  *
@@ -824,7 +827,7 @@ rasqal_new_aggregation_rowsource(rasqal_world *world, rasqal_query* query,
                                  raptor_sequence* exprs_seq,
                                  raptor_sequence* vars_seq)
 {
-  rasqal_aggregation_rowsource_context* con;
+  rasqal_aggregation_rowsource_context* con = NULL;
   int flags = 0;
   int size;
   int i;
@@ -897,6 +900,8 @@ rasqal_new_aggregation_rowsource(rasqal_world *world, rasqal_query* query,
     raptor_free_sequence(exprs_seq);
   if(vars_seq)
     raptor_free_sequence(vars_seq);
+  if(con)
+    RASQAL_FREE(rasqal_aggregation_rowsource_context*, con);
 
   return NULL;
 }
@@ -941,8 +946,8 @@ static const int const test1_output_rows[] =
 static const int const test2_output_rows[] =
 { 7, 6, };
 /* AVG(?x) GROUP BY ?x result */
-static const int const test3_output_rows[] =
-{ 1, 2, };
+static const double const test3_output_rows[] =
+{ 1.0, 2.0, };
 /* SAMPLE(?y) GROUP BY ?x result */
 static const int const test4_output_rows[] =
 { 2, 5, };
@@ -965,7 +970,9 @@ static const struct {
   int output_rows;
   const char* const *data;
   const int const *group_ids;
-  const int const *result_data;
+  rasqal_literal_type result_type;
+  const int const *result_int_data;
+  const double const *result_double_data;
   const char* const *result_string_data;
   rasqal_op op;
   const char* const expr_agg_vars[MAX_TEST_VARS];
@@ -976,7 +983,8 @@ static const struct {
    *   Output is 1 var (fake), 2 rows (1 per input group)
    * Expected result: [ ?fake => 3, ?fake => 5]
    */
-  {3, 3, 2, 1, 2, data_xyz_3_rows, test0_groupids, test0_output_rows,
+  {3, 3, 2, 1, 2, data_xyz_3_rows, test0_groupids,
+   RASQAL_LITERAL_INTEGER, test0_output_rows, NULL,
    NULL,
    RASQAL_EXPR_MAX, { "y" } },
 
@@ -984,7 +992,8 @@ static const struct {
    * Execute the aggregation part of SELECT (MIN(?x) AS ?fake) ... GROUP BY ?x
    * Expected result: [ ?fake => 1, ?fake => 2]
    */
-  {3, 3, 2, 1, 2, data_xyz_3_rows, test0_groupids, test1_output_rows,
+  {3, 3, 2, 1, 2, data_xyz_3_rows, test0_groupids, 
+   RASQAL_LITERAL_INTEGER, test1_output_rows, NULL,
    NULL,
    RASQAL_EXPR_MIN, { "x" } },
 
@@ -992,15 +1001,17 @@ static const struct {
    * Execute the aggregation part of SELECT (SUM(?z) AS ?fake) ... GROUP BY ?x
    * Expected result: [ ?fake => 7, ?fake => 6]
    */
-  {3, 3, 2, 1, 2, data_xyz_3_rows, test0_groupids, test2_output_rows,
+  {3, 3, 2, 1, 2, data_xyz_3_rows, test0_groupids,
+   RASQAL_LITERAL_INTEGER, test2_output_rows, NULL,
    NULL,
    RASQAL_EXPR_SUM, { "z" } },
 
   /*
    * Execute the aggregation part of SELECT (AVG(?x) AS ?fake) ... GROUP BY ?x
-   * Expected result: [ ?fake => 1, ?fake => 2]
+   * Expected result: [ ?fake => 1.0, ?fake => 2.0]
    */
-  {3, 3, 2, 1, 2, data_xyz_3_rows, test0_groupids, test3_output_rows,
+  {3, 3, 2, 1, 2, data_xyz_3_rows, test0_groupids, 
+   RASQAL_LITERAL_DECIMAL, NULL, test3_output_rows,
    NULL,
    RASQAL_EXPR_AVG, { "x" } },
 
@@ -1008,7 +1019,8 @@ static const struct {
    * Execute the aggregation part of SELECT (SAMPLE(?y) AS ?fake) ... GROUP BY ?x
    * Expected result: [ ?fake => 2, ?fake => 5]
    */
-  {3, 3, 2, 1, 2, data_xyz_3_rows, test0_groupids, test4_output_rows,
+  {3, 3, 2, 1, 2, data_xyz_3_rows, test0_groupids, 
+   RASQAL_LITERAL_INTEGER, test4_output_rows, NULL,
    NULL,
    RASQAL_EXPR_SAMPLE, { "y" } },
 
@@ -1016,7 +1028,8 @@ static const struct {
    * Execute the aggregation part of SELECT (GROUP_CONCAT(?z) AS ?fake) ... GROUP BY ?x
    * Expected result: [ ?fake => "3 4", ?fake => "6"]
    */
-  {3, 3, 2, 1, 2, data_xyz_3_rows, test0_groupids, NULL,
+  {3, 3, 2, 1, 2, data_xyz_3_rows, test0_groupids,
+   RASQAL_LITERAL_INTEGER, NULL, NULL,
    test5_output_rows,
    RASQAL_EXPR_GROUP_CONCAT, { "z" } }
 };
@@ -1085,7 +1098,9 @@ main(int argc, char *argv[])
     int output_rows_count = test_data[test_id].output_rows;
     int output_vars_count = test_data[test_id].output_vars;
     const int* input_group_ids = test_data[test_id].group_ids;
-    const int* result_int_data = test_data[test_id].result_data;
+    rasqal_literal_type expected_type = test_data[test_id].result_type;
+    const int* result_int_data = test_data[test_id].result_int_data;
+    const double* result_double_data = test_data[test_id].result_double_data;
     const char* const* result_string_data = test_data[test_id].result_string_data;
     rasqal_op op  = test_data[test_id].op;
     raptor_sequence* seq = NULL;
@@ -1096,7 +1111,7 @@ main(int argc, char *argv[])
     rasqal_variable* output_var;
     rasqal_expression* expr;
     int output_row_size = (input_vars_count + output_vars_count);
-    
+
     if(output_vars_count != 1) {
       fprintf(stderr,
               "%s: test %d expects %d variables which is not supported. Test skipped\n",
@@ -1132,7 +1147,7 @@ main(int argc, char *argv[])
       const unsigned char* var_name;
       
       for(vindex = 0;
-          (var_name = (const unsigned char*)test_data[test_id].expr_agg_vars[vindex] );
+          (var_name = RASQAL_GOOD_CAST(const unsigned char*, test_data[test_id].expr_agg_vars[vindex] ));
           vindex++) {
         rasqal_variable* v;
         rasqal_literal *l = NULL;
@@ -1140,8 +1155,11 @@ main(int argc, char *argv[])
 
         v = rasqal_variables_table_get_by_name(vt, RASQAL_VARIABLE_TYPE_NORMAL,
                                                var_name);
-        if(v)
+        /* returns SHARED pointer to variable */
+        if(v) {
+          v = rasqal_new_variable_from_variable(v);
           l = rasqal_new_variable_literal(world, v);
+        }
 
         if(l)
           e = rasqal_new_literal_expression(world, l);
@@ -1150,7 +1168,7 @@ main(int argc, char *argv[])
           raptor_sequence_push(expr_args_seq, e);
         else {
           fprintf(stderr, "%s: failed to create variable %s\n", program,
-                  (const char*)var_name);
+                  RASQAL_GOOD_CAST(const char*, var_name));
           failures++;
           goto tidy;
         }
@@ -1162,7 +1180,7 @@ main(int argc, char *argv[])
     output_var_name = RASQAL_MALLOC(char*, 5);
     memcpy(output_var_name, "fake", 5);
     output_var = rasqal_variables_table_add(vt, RASQAL_VARIABLE_TYPE_ANONYMOUS, 
-                                            (const unsigned char*)output_var_name, NULL);
+                                            RASQAL_GOOD_CAST(const unsigned char*, output_var_name), NULL);
 
     expr = make_test_expr(world, expr_args_seq, op);
     /* expr_args_seq is now owned by expr */
@@ -1176,13 +1194,17 @@ main(int argc, char *argv[])
     
     vars_seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
                                    (raptor_data_print_handler)rasqal_variable_print);
-    output_var = rasqal_new_variable_from_variable(output_var);
     raptor_sequence_push(vars_seq, output_var);
+    /* output_var is now owned by vars_seq */
+    output_var = NULL;
 
     rowsource = rasqal_new_aggregation_rowsource(world, query, input_rs,
                                                  exprs_seq, vars_seq);
-    /* exprs_seq, vars_seq and input_rs are now owned by rowsource */
-    exprs_seq = NULL; vars_seq = NULL; input_rs = NULL;
+    /* input_rs is now owned by rowsource */
+    input_rs = NULL;
+    /* these are no longer needed; agg rowsource made copies */
+    raptor_free_sequence(exprs_seq); exprs_seq = NULL;
+    raptor_free_sequence(vars_seq); vars_seq = NULL;
 
     if(!rowsource) {
       fprintf(stderr, "%s: failed to create aggregation rowsource\n", program);
@@ -1218,12 +1240,10 @@ main(int argc, char *argv[])
       goto tidy;
     }
 
-    if(result_int_data) {
+    if(result_int_data || result_double_data) {
       for(i = 0; i < output_rows_count; i++) {
         rasqal_row* row = (rasqal_row*)raptor_sequence_get_at(seq, i);
         rasqal_literal* value;
-        int integer;
-        int expected_integer = result_int_data[i];
         int vc;
         
         if(row->size != output_row_size) {
@@ -1252,25 +1272,46 @@ main(int argc, char *argv[])
             goto tidy;
           }
 
-          if(value->type != RASQAL_LITERAL_INTEGER) {
+          if(value->type != expected_type) {
             fprintf(stderr,
-                    "%s: test %d row #%d %s value #%d result is type %s expected integer\n",
+                    "%s: test %d row #%d %s value #%d result is type %s expected %s\n",
                     program, test_id, i, row_var->name, vc,
-                    rasqal_literal_type_label(value->type));
+                    rasqal_literal_type_label(value->type),
+                    rasqal_literal_type_label(expected_type));
             failures++;
             goto tidy;
           }
 
-          integer = rasqal_literal_as_integer(value, NULL);
+          if(expected_type == RASQAL_LITERAL_INTEGER) {
+            int expected_integer = result_int_data[i];
+            int integer;
 
-          if(integer != expected_integer) {
-            fprintf(stderr,
+            integer = rasqal_literal_as_integer(value, NULL);
+            
+            if(integer != expected_integer) {
+              fprintf(stderr,
                     "%s: test %d row #%d %s value #%d result is %d expected %d\n",
-                    program, test_id, i, row_var->name, vc,
-                    integer, expected_integer);
-            failures++;
-            goto tidy;
+                      program, test_id, i, row_var->name, vc,
+                      integer, expected_integer);
+              failures++;
+              goto tidy;
+            }
+          } else if(expected_type == RASQAL_LITERAL_DECIMAL) {
+            double expected_double = result_double_data[i];
+            double d;
+
+            d = rasqal_literal_as_double(value, NULL);
+            
+            if(d != expected_double) {
+              fprintf(stderr,
+                    "%s: test %d row #%d %s value #%d result is %f expected %f\n",
+                      program, test_id, i, row_var->name, vc,
+                      d, expected_double);
+              failures++;
+              goto tidy;
+            }
           }
+          
         }
         
       }
@@ -1321,7 +1362,7 @@ main(int argc, char *argv[])
 
           str = rasqal_literal_as_string(value);
 
-          if(strcmp((const char*)str, expected_string)) {
+          if(strcmp(RASQAL_GOOD_CAST(const char*, str), expected_string)) {
             fprintf(stderr,
                     "%s: test %d row #%d %s value #%d is %s expected %s\n",
                     program, test_id, i, row_var->name, vc,
